using System;
using System.Collections;

namespace ValutaWindowsApplication
{
	/// <summary>
	/// Summary description for ValutaBeregner.
	/// </summary>
	public class ValutaBeregner
	{
		private System.Collections.ArrayList valutakurser;

		// "samling" af metoder der skal kaldes ved ndring af et af valutakurs objekterne
		// Bruges som et gennerel "abonnement" p alle ndringer i sammlingen
		public event ValutakursndringsEvent	ValutakursndringsEventHandler;
		
		public ValutaBeregner()
		{
			valutakurser = new ArrayList();
			valutakurser.Add( new Valutakurs("Amerika","USD",609.2f));
			valutakurser.Add( new Valutakurs("Australien","AUD",426.52f));
			valutakurser.Add( new Valutakurs("Canada","CAD",469.18f));
			foreach (Valutakurs val in valutakurser)
			{	
				// valutaberegner skal have besked ved ndring af alle vakutakurser
				val.ValutakursndringsEventHandler += 
					new ValutakursndringsEvent(this.valutakursndringsBesked);
			}

		}

		// ndringsbesked fra eet af valutakurs - besked sendes vidre til
		// dem der "abonnere" p besked om gennerelle ndringer
		private void valutakursndringsBesked(Valutakurs valutakurs, string besked)
		{
			// send besked om bestemt valutakirs videre til "abonnenter"
			// der abonner gennerelt p alle
			if (this.ValutakursndringsEventHandler != null) // check metoder tilsluttet
				this.ValutakursndringsEventHandler(valutakurs, besked);	// vidresend
		}
		
		public Valutakurs OpretValutakurs(string land, string iso, float kurs)
		{
			Valutakurs ny = new Valutakurs(land, iso.ToUpper(), kurs);
			valutakurser.Add(ny);

			// valutaberegner skal have besked ved ndring af denne nye vakutakurs
			ny.ValutakursndringsEventHandler += 
				new ValutakursndringsEvent(this.valutakursndringsBesked);

			// besked til "abonnenter" om gennerelle ndringer
			if (this.ValutakursndringsEventHandler != null) // check metoder tilsluttet
				this.ValutakursndringsEventHandler(ny, "Valuta oprettet");	// besked om nye
			return ny;
		}

		public void Udskriv ()		// for hurtig aftestning af indhold
		{
			foreach (Valutakurs valkurs in valutakurser)
				System.Console.WriteLine(valkurs.ToString());
			System.Console.ReadLine();
		}

		public void Sorter ()			// normalt p iso
		{
			try
			{
				valutakurser.Sort();
			}
			catch (System.InvalidOperationException exception)
			{
				System.Console.WriteLine("Der kan ikke sorteres");
				System.Console.WriteLine("exception:"+exception.ToString());
			}
		
		}

		public void SorterPLand ()		// p land
		{
			valutakurser.Sort(new SortValutakursILandeorden());	// ekstern sorteringsmetode
		}

		public Valutakurs this [int index]		// indexer med index-nr til opslag  
		{										
			get							// aflsning eks.  valutakurs = valutaBeregner[10];
			{
				Valutakurs val = null;
				try 
				{
					val = (Valutakurs)valutakurser[index];
				}
				catch (System.IndexOutOfRangeException exception)
				{
					System.Console.WriteLine("Der er fejl i index:"+index);
					throw exception;
				}
				return val;
			}
/*
			set							// tilskrivning eks.  valutaBeregner[10] = valutakurs ;
			{							// her deaktiveret da dam ikke skal have lov
				valutakurser[index] = value;

			}
*/
		}
		public Valutakurs this [string iso]				// indexer med index-nr til opslag
		{
			get							// aflsning eks.  valutakurs = valutaBeregner["AUD"];
			{
				iso = iso.ToUpper();	// altid store bogstaver
				Valutakurs val = null;

				int index = 0;
				while (index < valutakurser.Count && val == null)
				{
					Valutakurs testVal = (Valutakurs) valutakurser[index];
					if (testVal.Iso.Equals(iso))
						val = testVal;
					index ++;
				}
				if (val == null)
				{
					throw new System.IndexOutOfRangeException("Iso findes ikke");
				}
				return val;
			}
/*
			set							// tilskrivning eks.  valutaBeregner["AUD"] = valutakurs ;
			{							// her deaktiveret da dam ikke skal have lov
				sgerutine som ovenfor og herefter hvis fundet
				valutakurser[index] = value;
			}
*/
		}
	}
}
