using System;
using System.Collections;

namespace ValutaWindowsApplication
{
	// Fejlhndterings klasse vedr. kurser
	public class FejlKursException : System.ApplicationException
	{
		public FejlKursException (string message)			// inner-exception er fjernet igen fra
			: base (message)								// tidligere demo af dette da det
		{}													// egentlig ikke er relevant her
	}
	
	// Ekstra sammenligningsmetode for f.eks sorting 
	public class SortValutakursILandeorden : System.Collections.IComparer
	{
		public int Compare (Object objLeft, Object objRight)
		{
			Valutakurs kursLeft = (Valutakurs)objLeft; 
			Valutakurs kursRight = (Valutakurs)objRight; 
			return kursLeft.Navn.CompareTo(kursRight.Navn);
		}
	}

	// metode specifikation for modtagelse af besked om kursndring
	public delegate void ValutakursndringsEvent (Valutakurs valutakurs, string besked);

	public class Valutakurs : System.IComparable
	{
		private string navn;
		private string iso;
		private float kurs;

		// "samling" af metoder der skal kaldes ved kursndring af dette valutakurs objekt
		public event ValutakursndringsEvent	ValutakursndringsEventHandler;

		public Valutakurs(string navn, string iso, float kurs)
		{
			this.navn = navn;
			this.iso = iso;
			this.kurs = kurs;
		}
		public string Navn
		{
			get
			{
				return navn;
			}
		}
		public string Iso
		{
			get
			{
				return iso;
			}
		}
		public float Kurs
		{
			get
			{
				return kurs;
			}
			set
			{
				if (value <= 0)
				{
					throw new FejlKursException("Kurs ikke gyldig: "+value);
				}
				else if (kurs != value)		// behandl kun ndring
				{
					string besked;

					if (value < kurs) besked = "Kurs faldet fra: "+kurs.ToString();
					else			  besked = "Kurs steget fra: "+kurs.ToString();
					// ovenstende if-else tildeling til besked kan kortfattet skrives som
					//	besked = (value < kurs)	?	"Kurs faldet fra: "+kurs.ToString()
					//							:	"Kurs steget fra: "+kurs.ToString();
					// eller endnu kortere - husk parantesen om det der indgr i test tildelingen
					// besked = "Kurs "+( (value < kurs) ? "faldet" : "steget" )+" fra "+kurs.ToString();

					kurs = value;	// ndring af kurs

					// send besked til "abonnenter"
					if (this.ValutakursndringsEventHandler != null) // check metoder tilsluttet
						this.ValutakursndringsEventHandler(this, besked);
				}
			}
		}
						
		public override string ToString()
		{
			return "navn:"+navn+" iso:"+iso+" kurs:"+kurs;
		}

		#region IComparable Members		standart sammenligningsmetode for Valutakurs

		public int CompareTo(object obj)
		{
			Valutakurs andet = (Valutakurs) obj;
			return this.iso.CompareTo(andet.iso) ;
		}

		#endregion
	}
}
